<?php

/**
 * Class PlagScanApi
 * @author Ingmar Szmais <iszmais@databay.de>
 * @since 20.08.19
 */
class API
{

    protected $HOST = 'https://api.plagscan.com/';
    protected $VERSION = 'v3';

    /**
     * @var string Base URL
     */
    protected $connection;

    /**
     * @var API
     */
    protected static $instance;

    /**
     * @param $host
     * @param string $version
     * @return API
     */
    public static function getInstance($host, $version = 'v3') : API
    {
        if (self::$instance === null) {
            self::$instance = new self($host, $version);
        }
        return self::$instance;
    }

    /**
     * PlagScanApi constructor.
     * @param $host
     * @param $version
     */
    public function __construct($host, $version)
    {
        $this->HOST = $host;
        $this->VERSION = $version;
        $this->connection = $this->HOST . $this->VERSION;
    }

    /**
     * Make a HTTP request to the API
     *
     * @param string $endPoint
     * @param string $requestType
     * @param string $accessToken
     * @param array $data
     *
     * @return array|string
     */
    public function request(string $endPoint, string $requestType, string $accessToken = '', array $data = [])
    {
        $ch = curl_init();
        $url = $this->connection . '/' . $endPoint;
        if ($accessToken) {
            $url .= '?access_token=' . $accessToken;
        }
        if ($requestType === 'GET' || $requestType === 'PATCH') {
            foreach ($data as $key => $value) {
                $url .= '&' . $key . '=' . $value;
            }
            if (strpos($url, '?') === false) {
                $url = preg_replace('/\&/', '?', $url, 1);
            }
            $data = [];
        }
        $curlopt = [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_CUSTOMREQUEST => $requestType,
        ];
        if (!empty($data)) {
            if(isset($data['fileUpload'])){
                $curlopt = $this->createFileDataParams($data) + $curlopt;
            } else {
                $curlopt = [CURLOPT_POSTFIELDS => $data] + $curlopt;
            }
        }
        curl_setopt_array($ch, $curlopt);
        $response = curl_exec($ch);
        curl_close($ch);
        $response = json_decode($response, true) ?? $response;
        if (isset($response['data'])) {
            return $response['data'];
        }
        if (isset( $response['error'])) {
            return $response['error']['message'];
        }
        return $response;
    }

    /**
     * Creates a string for fileuploads.
     *
     * @param array $data
     * @return array
     */
    private function createFileDataParams(array $data){
        $string = "";
        $id = uniqid();
        foreach ($data as $key => $param) {
            if ($key !== 'fileUpload') {
                $string .=
                    "---------------$id\r\n"
                    . "Content-Disposition: form-data; name=\"$key\"\r\n"
                    . "\r\n$param\r\n";
            }
        }
        $fn = end(explode('/',$data['fileUpload']));
        $mime = mime_content_type($data['fileUpload']);
        $content = file_get_contents($data['fileUpload']);
        $string .=
            "---------------$id\r\n"
            . "Content-Disposition: form-data; name=\"fileUpload\"; filename=\"$fn\"\r\n"
            . "Content-Type: $mime\r\n"
            . "Content-Transfer-Encoding: binary\r\n"
            . "\r\n$content\r\n"
            . "---------------$id--\r\n";
        return [
            CURLOPT_POSTFIELDS => $string,
            CURLOPT_HTTPHEADER => [
                "Content-Type: multipart/form-data; boundary=-------------$id",
                "Content-Length: " . strlen($string)
            ]
        ];
    }
}