<?php

namespace PlagScanApi;
/**
 * Class Ping
 * @author Ingmar Szmais <iszmais@databay.de>
 * @since 20.08.19
 */
class Base
{
    /**
     * @var \API
     */
    protected $handler;

    /**
     * @var array
     */
    protected $accessToken;

    /**
     * Ping constructor.
     * @param $host
     * @param string $version
     */
    public function __construct($host, $version = 'v3')
    {
        require_once __DIR__ . '/../API.php';
        $this->handler = \API::getInstance($host, $version);
        $this->createAccessToken();
    }

    /**
     * @return array|string
     */
    public function ping()
    {
        return $this->handler->request('ping', 'GET');
    }

    /**
     * @return array|string
     */
    public function checkCallbackConfiguration()
    {
        return $this->handler->request('checkcallback', 'PUT', $this->accessToken);
    }

    /**
     * @param string $client_id
     * @param string $client_secret
     * @return array|string
     */
    public function createAccessToken(string $client_id = '', string $client_secret = '')
    {
        $settings = new \ilSetting('psx');
        if($client_id === ''){
            $client_id = $settings->get('psx_clientid');
        }
        if($client_secret === ''){
            $client_secret = $settings->get('psx_apikey');
        }

        $response = $this->handler->request('token', 'POST', '', [
            'client_id' => $client_id,
            'client_secret' => $client_secret,
        ]);

        if (isset($response['access_token'])) {
            $this->accessToken = $response['access_token'];
        }

        return $response;
    }
}